@@ -8,9 +8,11 @@ from ..operations import flush, purge_cache
 
 @click.command("flush")
 @profiler_option()
+@click.option(
+    "--with-cache", is_flag=True, default=False, help="Also delete all cache",
+)
 @global_options()
 @pass_context
 def cli(ctx, **kwargs):
-    """Purge cache, remove ALL TABLES from the target database and recreate them"""
-    purge_cache(ctx)
+    """Remove ALL TABLES from the target database and recreate them"""
     flush(ctx)
