@@ -8,23 +8,25 @@ import sys
 import threading
 from contextlib import contextmanager
 
-from marshmallow import fields, post_dump
-from marshmallow_sqlalchemy import ModelSchema
 from sqlalchemy import MetaData, create_engine, event, func, inspect
 from sqlalchemy.engine.url import make_url
 from sqlalchemy.ext.automap import automap_base, generate_relationship
-from sqlalchemy.orm import mapper
 from sqlalchemy.schema import conv
 from sqlalchemy.sql.expression import select
 from sqlalchemy.types import Text
 
+from . import VERSION
 from .configuration import DEFAULT_CONFIG
-from .marshmallow_schema import register_new_schema
 from .models import BaseDeclarativeMeta, BaseModel
 from .query import BaseQuery, QueryProperty
 from .session import SessionProperty
-from .utils import (aslist, cached_property, create_directory,
-                    generate_valid_index_name, to_unicode)
+from .utils import (
+    aslist,
+    cached_property,
+    create_directory,
+    generate_valid_index_name,
+    to_unicode,
+)
 
 try:
     from easy_profile import SessionProfiler, StreamReporter
@@ -88,7 +90,6 @@ class Database(object):
 
         event.listen(self.engine, "before_cursor_execute", self._before_custor_execute)
         event.listen(self.engine, "after_cursor_execute", self._after_custor_execute)
-        event.listen(mapper, "after_configured", self._configure_serialization)
 
     @cached_property
     def cache_dir(self):
@@ -98,7 +99,7 @@ class Database(object):
             )
         else:
             db_cache_dir = os.path.join(self.uri.drivername, self.uri.database)
-        _cache_dir = os.path.join(self.global_cache_dir, db_cache_dir)
+        _cache_dir = os.path.join(self.global_cache_dir, VERSION, db_cache_dir)
         create_directory(_cache_dir)
         return _cache_dir
 
@@ -167,44 +168,41 @@ class Database(object):
     def reflect(self, bind=None):
         """Reflect metadata from database"""
         if not self._reflected:
+            reflect = True
             if bind is None:
                 bind = self.engine
             if self.enable_cache and self.cached_metadata:
-                self.Model.prepare(bind)
-            else:
-                self.Model.prepare(
-                    bind,
-                    reflect=True,
-                    name_for_scalar_relationship=self._name_for_scalar_relationship,
-                    name_for_collection_relationship=self._name_for_collection_relationship,
-                    generate_relationship=self._gen_relationship,
-                )
+                reflect = False
 
-                for table in self.tables.values():
-                    for constraint in table.constraints:
-                        if constraint.name:
-                            constraint.name = conv(constraint.name)
+            self.Model.prepare(
+                bind,
+                reflect=reflect,
+                name_for_scalar_relationship=self._name_for_scalar_relationship,
+                name_for_collection_relationship=self._name_for_collection_relationship,
+                generate_relationship=self._gen_relationship,
+            )
 
-                for index in self.get_all_indexes():
-                    index.name = conv(
-                        generate_valid_index_name(index, self.engine.dialect)
-                    )
-                    mysql_length = {}
+            for table in self.tables.values():
+                for constraint in table.constraints:
+                    if constraint.name:
+                        constraint.name = conv(constraint.name)
 
-                    if self.engine.dialect.name == "mysql":
-                        for column in list(index.columns):
-                            if isinstance(column.type, Text):
-                                mysql_length[column.name] = _MYSQL_LENGHT_TEXT_INDEX_COLUMN
-                        if len(list(index.columns)) == 1 and mysql_length:
-                            mysql_length = mysql_length[next(iter(mysql_length))]
-                        if mysql_length:
-                            index.kwargs['mysql_length'] = mysql_length
+            for index in self.get_all_indexes():
+                index.name = conv(generate_valid_index_name(index, self.engine.dialect))
+                mysql_length = {}
 
-                if self.enable_cache:
-                    with open(
-                        os.path.join(self.cached_metadata_path), "wb"
-                    ) as cache_file:
-                        pickle.dump(self.metadata, cache_file)
+                if self.engine.dialect.name == "mysql":
+                    for column in list(index.columns):
+                        if isinstance(column.type, Text):
+                            mysql_length[column.name] = _MYSQL_LENGHT_TEXT_INDEX_COLUMN
+                    if len(list(index.columns)) == 1 and mysql_length:
+                        mysql_length = mysql_length[next(iter(mysql_length))]
+                    if mysql_length:
+                        index.kwargs["mysql_length"] = mysql_length
+
+            if self.enable_cache and not self.cached_metadata:
+                with open(os.path.join(self.cached_metadata_path), "wb") as cache_file:
+                    pickle.dump(self.metadata, cache_file)
 
             self._reflected = True
 
@@ -370,71 +368,11 @@ class Database(object):
     ):
         kw["lazy"] = "noload"
         kw["cascade"] = "all"
+        kw["post_update"] = True
         return generate_relationship(
             base, direction, return_fn, attrname, local_cls, referred_cls, **kw
         )
 
-    def _configure_serialization(self):
-        module_basename = ".".join(
-            self.__class__.__module__.split(".")[:-1] + ["marshmallow_schema"]
-        )
-
-        for class_ in self.models.values():
-
-            class Meta(object):
-                model = class_
-                transient = True
-                include_fk = True
-
-            attrs = {"Meta": Meta}
-            attrs["__module__"] = module_basename
-            schema_class_name = "%s_%s_marshmallow_schema" % (id(self), class_.__name__)
-
-            relationship_keys = class_.__mapper__.relationships.keys()
-
-            for keyname in relationship_keys:
-                relationship = class_.__mapper__.relationships[keyname]
-                target_name = relationship.target.name
-                if target_name in self.models:
-
-                    many = relationship.uselist
-
-                    target_schema_class_name = "%s_%s_marshmallow_schema" % (
-                        id(self),
-                        self.models[target_name].__name__,
-                    )
-                    target_schema_class_fullname = "%s.%s" % (
-                        attrs["__module__"],
-                        target_schema_class_name,
-                    )
-                    exclude_keys = []
-
-                    if target_name == class_.__name__:
-                        exclude_keys = [keyname]
-                        attrs[keyname] = fields.Nested(
-                            "self",
-                            name=keyname,
-                            many=many,
-                            exclude=exclude_keys,
-                            default=None,
-                        )
-                    else:
-                        attrs[keyname] = fields.Nested(
-                            target_schema_class_fullname,
-                            name=keyname,
-                            many=many,
-                            exclude=exclude_keys,
-                        )
-
-            schema_class = type(schema_class_name, (BaseSchema,), attrs)
-            register_new_schema(schema_class)
-            setattr(class_, "__marshmallow__", schema_class)
-
-        for class_ in self.models.values():
-            for key, field in class_.__marshmallow__._declared_fields.items():
-                if field:
-                    field.allow_none = True
-
     def _echo_statement(self, stm):
         text = to_unicode(stm)
         text = re.sub(r";", ";\n", text)
@@ -460,7 +398,10 @@ class Database(object):
     ):
         if self.echo_sql:
             if conn.engine.dialect.name == "mysql":
-                self._echo_statement(cursor._last_executed)
+                if hasattr(cursor, "_executed"):
+                    self._echo_statement(cursor._executed)
+                else:
+                    self._echo_statement(cursor._last_executed)
             if conn.engine.dialect.name == "postgresql":
                 self._echo_statement(cursor.query)
 
@@ -483,20 +424,6 @@ class Database(object):
         return "<%s engine=%r>" % (self.__class__.__name__, engine)
 
 
-class BaseSchema(ModelSchema):
-    SKIP_VALUES = set([None])
-
-    @post_dump
-    def remove_skip_values(self, data, **kwargs):
-        return dict(
-            (
-                (key, value)
-                for key, value in data.items()
-                if value is not None or not isinstance(self.fields[key], fields.Nested)
-            )
-        )
-
-
 class EngineConnector(object):
     def __init__(self, db, connect_timeout=3):
         self._db = db
